@@ -1,11 +1,11 @@
 // 2004-03-02  Petur Runolfsson  <peturr02@ru.is>
 
-// Copyright (C) 2004 Free Software Foundation, Inc.
+// Copyright (C) 2004, 2009 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,14 +14,12 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 27.4.3 fpos
 
 #include <iterator>
-#include <sstream>
 #include <testsuite_hooks.h>
 
 // libstdc++/14320
@@ -30,12 +28,11 @@ void test01()
   using namespace std;
   bool test __attribute__((unused)) = true;
 
-  typedef istreambuf_iterator<char>::difference_type Distance;
-	
-  Distance d;
-  istringstream in("5");
-  in >> d;
-  VERIFY( d == 5 );
+  typedef std::istreambuf_iterator<char>::difference_type Distance;
+  Distance d = 2;
+  Distance e = 3;
+  d *= e;
+  VERIFY( static_cast<int>(d) == 6 );
 }
 
 int main()
